package com.example.fingerprint_tlk1_uart_demo;

import android.content.Context;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.Fragment;

import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.rscja.deviceapi.FingerprintWithTLK1NC;
import com.rscja.utility.StringUtility;


public class AuthenticationFragment extends Fragment {

    private MainActivity mActivity;

    private EditText etPageId;
    private Button btnAuthentication;
    private TextView tvMsg;

    private int pageID;

    private static final int GET_IMAGE_FAIL = -1;
    private static final int STORE_BUFFER_FAIL = -2;
    private static final int STORE_FLASH_FAIL = -3;
    private static final int FAIL = -4;
    private static final int SUCC = 1;
    private Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            mActivity.mProgressDialog.dismiss();
            switch (msg.what) {
                case SUCC:
                    tvMsg.setText("Authentication success");
                    tvMsg.setTextColor(ActivityCompat.getColor(getContext(), R.color.green_0C7910));
                    showToast("Authentication success");
                    break;
                case FAIL:
                    tvMsg.setText("Authentication failure");
                    tvMsg.setTextColor(Color.RED);
                    showToast("Authentication failure");
                    break;
                case GET_IMAGE_FAIL:
                    tvMsg.setText("Authentication failure\n\nfailure to capture fingerprint image");
                    tvMsg.setTextColor(Color.RED);
                    showToast("Authentication failure\n\nfailure to capture fingerprint image");
                    break;
                case STORE_BUFFER_FAIL:
                    tvMsg.setText("Authentication failure\n\nfailed to store to buffer");
                    tvMsg.setTextColor(Color.RED);
                    showToast("Authentication failure\n\nfailed to store to buffer");
                    break;
                case STORE_FLASH_FAIL:
                    tvMsg.setText("Authentication failure\n\nfailed to store to flash");
                    tvMsg.setTextColor(Color.RED);
                    showToast("Authentication failure\n\nfailed to store to flash");
                    break;
            }
        }
    };

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_authentication, container, false);
        init(view);
        return view;
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mActivity = (MainActivity) getActivity();
    }

    @Override
    public void onDestroyView() {
        mHandler.removeCallbacksAndMessages(null);
        super.onDestroyView();
    }

    private void init(View view) {
        etPageId = view.findViewById(R.id.etPageId);
        tvMsg = view.findViewById(R.id.tvMsg);
        btnAuthentication = view.findViewById(R.id.btnAuthentication);
        btnAuthentication.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                authentication();
            }
        });
    }

    boolean isRuning=false;
    private void authentication() {
        String pageIdStr = etPageId.getText().toString();
        if(!mActivity.checkPageID(pageIdStr)) {
            return;
        }

        pageID = StringUtility.string2Int(pageIdStr, -1);
        if (pageID < 0 || pageID > 127) {
            showToast("Page ID is error, from 0 to 127");
            return;
        }

        tvMsg.setText("Start authentication...");
        mActivity.mProgressDialog.setMessage("Start authentication...");
        mActivity.mProgressDialog.setCancelable(false);
        mActivity.mProgressDialog.show();
        isRuning=true;
        mActivity.mExecutorService.execute(new Runnable() {
            @Override
            public void run() {
                while (isRuning){
                    if (mActivity.mFingerprintWithTLK1NC.fingerGETImage() > 0) {
                        if (mActivity.mFingerprintWithTLK1NC.fingerStoreCharToBuffer(FingerprintWithTLK1NC.BufferEnum.B1)) {
                            if(mActivity.mFingerprintWithTLK1NC.fingerPKTemplate2(FingerprintWithTLK1NC.BufferEnum.B1, pageID)) {
                                mHandler.sendEmptyMessage(SUCC);
                            } else {
                                mHandler.sendEmptyMessage(FAIL);
                            }
                        } else {
                            mHandler.sendEmptyMessage(STORE_BUFFER_FAIL);
                        }
                        isRuning=false;
                        return;
                    } else {
                        //mHandler.sendEmptyMessage(GET_IMAGE_FAIL);
                    }
                }
            }
        });
    }
    public void showToast(String text) {
        Toast mToast = Toast.makeText(getContext(), text, Toast.LENGTH_SHORT);
        mToast.show();
    }
}
